@@ -56,8 +56,10 @@ from telegram import (
     Voice,
     WebAppData,
 )
+from telegram._utils.datetime import UTC
 from telegram.constants import ChatAction, ParseMode
 from telegram.ext import Defaults
+from telegram.warnings import PTBDeprecationWarning
 from tests._passport.test_passport import RAW_PASSPORT_DATA
 from tests.auxil.bot_method_checks import (
     check_defaults_handling,
@@ -83,7 +85,6 @@ def message(bot):
 
 
 @pytest.fixture(
-    scope="function",
     params=[
         {"forward_from": User(99, "forward_user", False), "forward_date": datetime.utcnow()},
         {
@@ -322,10 +323,11 @@ class TestMessageBase:
         {"length": 9, "offset": 101, "type": "strikethrough"},
         {"length": 10, "offset": 129, "type": "pre", "language": "python"},
         {"length": 7, "offset": 141, "type": "spoiler"},
+        {"length": 2, "offset": 150, "type": "custom_emoji", "custom_emoji_id": "1"},
     ]
     test_text_v2 = (
         r"Test for <bold, ita_lic, \`code, links, text-mention and `\pre. "
-        "http://google.com and bold nested in strk>trgh nested in italic. Python pre. Spoiled."
+        "http://google.com and bold nested in strk>trgh nested in italic. Python pre. Spoiled. 👍."
     )
     test_message = Message(
         message_id=1,
@@ -366,6 +368,46 @@ class TestMessageWithoutRequest(TestMessageBase):
         for slot in new.__slots__:
             assert not isinstance(new[slot], dict)
 
+    def test_de_json_localization(self, bot, raw_bot, tz_bot):
+        json_dict = {
+            "message_id": 12,
+            "from_user": None,
+            "date": int(datetime.now().timestamp()),
+            "chat": None,
+            "edit_date": int(datetime.now().timestamp()),
+            "forward_date": int(datetime.now().timestamp()),
+        }
+
+        message_raw = Message.de_json(json_dict, raw_bot)
+        message_bot = Message.de_json(json_dict, bot)
+        message_tz = Message.de_json(json_dict, tz_bot)
+
+        # comparing utcoffsets because comparing timezones is unpredicatable
+        date_offset = message_tz.date.utcoffset()
+        date_tz_bot_offset = tz_bot.defaults.tzinfo.utcoffset(message_tz.date.replace(tzinfo=None))
+
+        edit_date_offset = message_tz.edit_date.utcoffset()
+        edit_date_tz_bot_offset = tz_bot.defaults.tzinfo.utcoffset(
+            message_tz.edit_date.replace(tzinfo=None)
+        )
+
+        forward_date_offset = message_tz.forward_date.utcoffset()
+        forward_date_tz_bot_offset = tz_bot.defaults.tzinfo.utcoffset(
+            message_tz.forward_date.replace(tzinfo=None)
+        )
+
+        assert message_raw.date.tzinfo == UTC
+        assert message_bot.date.tzinfo == UTC
+        assert date_offset == date_tz_bot_offset
+
+        assert message_raw.edit_date.tzinfo == UTC
+        assert message_bot.edit_date.tzinfo == UTC
+        assert edit_date_offset == edit_date_tz_bot_offset
+
+        assert message_raw.forward_date.tzinfo == UTC
+        assert message_bot.forward_date.tzinfo == UTC
+        assert forward_date_offset == forward_date_tz_bot_offset
+
     def test_equality(self):
         id_ = 1
         a = Message(id_, self.date, self.chat, from_user=self.from_user)
@@ -473,7 +515,8 @@ class TestMessageWithoutRequest(TestMessageBase):
             r"<pre>`\pre</pre>. http://google.com "
             "and <i>bold <b>nested in <s>strk&gt;trgh</s> nested in</b> italic</i>. "
             '<pre><code class="python">Python pre</code></pre>. '
-            '<span class="tg-spoiler">Spoiled</span>.'
+            '<span class="tg-spoiler">Spoiled</span>. '
+            '<tg-emoji emoji-id="1">👍</tg-emoji>.'
         )
         text_html = self.test_message_v2.text_html
         assert text_html == test_html_string
@@ -492,7 +535,8 @@ class TestMessageWithoutRequest(TestMessageBase):
             r'<pre>`\pre</pre>. <a href="http://google.com">http://google.com</a> '
             "and <i>bold <b>nested in <s>strk&gt;trgh</s> nested in</b> italic</i>. "
             '<pre><code class="python">Python pre</code></pre>. '
-            '<span class="tg-spoiler">Spoiled</span>.'
+            '<span class="tg-spoiler">Spoiled</span>. '
+            '<tg-emoji emoji-id="1">👍</tg-emoji>.'
         )
         text_html = self.test_message_v2.text_html_urled
         assert text_html == test_html_string
@@ -513,7 +557,7 @@ class TestMessageWithoutRequest(TestMessageBase):
             "[links](http://github.com/abc\\\\\\)def), "
             "[text\\-mention](tg://user?id=123456789) and ```\\`\\\\pre```\\. "
             r"http://google\.com and _bold *nested in ~strk\>trgh~ nested in* italic_\. "
-            "```python\nPython pre```\\. ||Spoiled||\\."
+            "```python\nPython pre```\\. ||Spoiled||\\. ![👍](tg://emoji?id=1)\\."
         )
         text_markdown = self.test_message_v2.text_markdown_v2
         assert text_markdown == test_md_string
@@ -524,19 +568,19 @@ class TestMessageWithoutRequest(TestMessageBase):
             MessageEntity(MessageEntity.BOLD, offset=0, length=4),
             MessageEntity(MessageEntity.ITALIC, offset=0, length=4),
         ]
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError, match="Nested entities are not supported for"):
             assert message.text_markdown
 
         message.entities = [MessageEntity(MessageEntity.UNDERLINE, offset=0, length=4)]
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError, match="Underline entities are not supported for"):
             message.text_markdown
 
         message.entities = [MessageEntity(MessageEntity.STRIKETHROUGH, offset=0, length=4)]
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError, match="Strikethrough entities are not supported for"):
             message.text_markdown
 
         message.entities = [MessageEntity(MessageEntity.SPOILER, offset=0, length=4)]
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError, match="Spoiler entities are not supported for"):
             message.text_markdown
 
         message.entities = []
@@ -563,7 +607,8 @@ class TestMessageWithoutRequest(TestMessageBase):
             "[links](http://github.com/abc\\\\\\)def), "
             "[text\\-mention](tg://user?id=123456789) and ```\\`\\\\pre```\\. "
             r"[http://google\.com](http://google.com) and _bold *nested in ~strk\>trgh~ "
-            "nested in* italic_\\. ```python\nPython pre```\\. ||Spoiled||\\."
+            "nested in* italic_\\. ```python\nPython pre```\\. ||Spoiled||\\. "
+            "![👍](tg://emoji?id=1)\\."
         )
         text_markdown = self.test_message_v2.text_markdown_v2_urled
         assert text_markdown == test_md_string
@@ -594,17 +639,72 @@ class TestMessageWithoutRequest(TestMessageBase):
     @pytest.mark.parametrize(
         "type_",
         argvalues=[
-            "text_html",
-            "text_html_urled",
             "text_markdown",
             "text_markdown_urled",
+        ],
+    )
+    def test_text_custom_emoji_md_v1(self, type_, recwarn):
+        text = "Look a custom emoji: 😎"
+        expected = "Look a custom emoji: 😎"
+        emoji_entity = MessageEntity(
+            type=MessageEntity.CUSTOM_EMOJI,
+            offset=21,
+            length=2,
+            custom_emoji_id="5472409228461217725",
+        )
+        message = Message(
+            1,
+            from_user=self.from_user,
+            date=self.date,
+            chat=self.chat,
+            text=text,
+            entities=[emoji_entity],
+        )
+        assert expected == getattr(message, type_)
+
+        assert len(recwarn) == 1
+        assert recwarn[0].category is PTBDeprecationWarning
+        assert str(recwarn[0].message).startswith(
+            "Custom emoji entities are not supported for Markdown version 1"
+        )
+        assert recwarn[0].filename == __file__
+
+    @pytest.mark.parametrize(
+        "type_",
+        argvalues=[
             "text_markdown_v2",
             "text_markdown_v2_urled",
         ],
     )
-    def test_text_custom_emoji(self, type_):
+    def test_text_custom_emoji_md_v2(self, type_):
         text = "Look a custom emoji: 😎"
-        expected = "Look a custom emoji: 😎"
+        expected = "Look a custom emoji: ![😎](tg://emoji?id=5472409228461217725)"
+        emoji_entity = MessageEntity(
+            type=MessageEntity.CUSTOM_EMOJI,
+            offset=21,
+            length=2,
+            custom_emoji_id="5472409228461217725",
+        )
+        message = Message(
+            1,
+            from_user=self.from_user,
+            date=self.date,
+            chat=self.chat,
+            text=text,
+            entities=[emoji_entity],
+        )
+        assert expected == message[type_]
+
+    @pytest.mark.parametrize(
+        "type_",
+        argvalues=[
+            "text_html",
+            "text_html_urled",
+        ],
+    )
+    def test_text_custom_emoji_html(self, type_):
+        text = "Look a custom emoji: 😎"
+        expected = 'Look a custom emoji: <tg-emoji emoji-id="5472409228461217725">😎</tg-emoji>'
         emoji_entity = MessageEntity(
             type=MessageEntity.CUSTOM_EMOJI,
             offset=21,
@@ -630,7 +730,8 @@ class TestMessageWithoutRequest(TestMessageBase):
             r"<pre>`\pre</pre>. http://google.com "
             "and <i>bold <b>nested in <s>strk&gt;trgh</s> nested in</b> italic</i>. "
             '<pre><code class="python">Python pre</code></pre>. '
-            '<span class="tg-spoiler">Spoiled</span>.'
+            '<span class="tg-spoiler">Spoiled</span>. '
+            '<tg-emoji emoji-id="1">👍</tg-emoji>.'
         )
         caption_html = self.test_message_v2.caption_html
         assert caption_html == test_html_string
@@ -649,7 +750,8 @@ class TestMessageWithoutRequest(TestMessageBase):
             r'<pre>`\pre</pre>. <a href="http://google.com">http://google.com</a> '
             "and <i>bold <b>nested in <s>strk&gt;trgh</s> nested in</b> italic</i>. "
             '<pre><code class="python">Python pre</code></pre>. '
-            '<span class="tg-spoiler">Spoiled</span>.'
+            '<span class="tg-spoiler">Spoiled</span>. '
+            '<tg-emoji emoji-id="1">👍</tg-emoji>.'
         )
         caption_html = self.test_message_v2.caption_html_urled
         assert caption_html == test_html_string
@@ -670,7 +772,7 @@ class TestMessageWithoutRequest(TestMessageBase):
             "[links](http://github.com/abc\\\\\\)def), "
             "[text\\-mention](tg://user?id=123456789) and ```\\`\\\\pre```\\. "
             r"http://google\.com and _bold *nested in ~strk\>trgh~ nested in* italic_\. "
-            "```python\nPython pre```\\. ||Spoiled||\\."
+            "```python\nPython pre```\\. ||Spoiled||\\. ![👍](tg://emoji?id=1)\\."
         )
         caption_markdown = self.test_message_v2.caption_markdown_v2
         assert caption_markdown == test_md_string
@@ -697,7 +799,8 @@ class TestMessageWithoutRequest(TestMessageBase):
             "[links](http://github.com/abc\\\\\\)def), "
             "[text\\-mention](tg://user?id=123456789) and ```\\`\\\\pre```\\. "
             r"[http://google\.com](http://google.com) and _bold *nested in ~strk\>trgh~ "
-            "nested in* italic_\\. ```python\nPython pre```\\. ||Spoiled||\\."
+            "nested in* italic_\\. ```python\nPython pre```\\. ||Spoiled||\\. "
+            "![👍](tg://emoji?id=1)\\."
         )
         caption_markdown = self.test_message_v2.caption_markdown_v2_urled
         assert caption_markdown == test_md_string
@@ -733,17 +836,72 @@ class TestMessageWithoutRequest(TestMessageBase):
     @pytest.mark.parametrize(
         "type_",
         argvalues=[
-            "caption_html",
-            "caption_html_urled",
             "caption_markdown",
             "caption_markdown_urled",
+        ],
+    )
+    def test_caption_custom_emoji_md_v1(self, type_, recwarn):
+        caption = "Look a custom emoji: 😎"
+        expected = "Look a custom emoji: 😎"
+        emoji_entity = MessageEntity(
+            type=MessageEntity.CUSTOM_EMOJI,
+            offset=21,
+            length=2,
+            custom_emoji_id="5472409228461217725",
+        )
+        message = Message(
+            1,
+            from_user=self.from_user,
+            date=self.date,
+            chat=self.chat,
+            caption=caption,
+            caption_entities=[emoji_entity],
+        )
+        assert expected == getattr(message, type_)
+
+        assert len(recwarn) == 1
+        assert recwarn[0].category is PTBDeprecationWarning
+        assert str(recwarn[0].message).startswith(
+            "Custom emoji entities are not supported for Markdown version 1"
+        )
+        assert recwarn[0].filename == __file__
+
+    @pytest.mark.parametrize(
+        "type_",
+        argvalues=[
             "caption_markdown_v2",
             "caption_markdown_v2_urled",
         ],
     )
-    def test_caption_custom_emoji(self, type_):
+    def test_caption_custom_emoji_md_v2(self, type_):
         caption = "Look a custom emoji: 😎"
-        expected = "Look a custom emoji: 😎"
+        expected = "Look a custom emoji: ![😎](tg://emoji?id=5472409228461217725)"
+        emoji_entity = MessageEntity(
+            type=MessageEntity.CUSTOM_EMOJI,
+            offset=21,
+            length=2,
+            custom_emoji_id="5472409228461217725",
+        )
+        message = Message(
+            1,
+            from_user=self.from_user,
+            date=self.date,
+            chat=self.chat,
+            caption=caption,
+            caption_entities=[emoji_entity],
+        )
+        assert expected == message[type_]
+
+    @pytest.mark.parametrize(
+        "type_",
+        argvalues=[
+            "caption_html",
+            "caption_html_urled",
+        ],
+    )
+    def test_caption_custom_emoji_html(self, type_):
+        caption = "Look a custom emoji: 😎"
+        expected = 'Look a custom emoji: <tg-emoji emoji-id="5472409228461217725">😎</tg-emoji>'
         emoji_entity = MessageEntity(
             type=MessageEntity.CUSTOM_EMOJI,
             offset=21,
@@ -783,7 +941,7 @@ class TestMessageWithoutRequest(TestMessageBase):
         assert message.link == f"https://t.me/{message.chat.username}/{message.message_id}"
 
     @pytest.mark.parametrize(
-        "type_, id_", argvalues=[(Chat.CHANNEL, -1003), (Chat.SUPERGROUP, -1003)]
+        ("type_", "id_"), argvalues=[(Chat.CHANNEL, -1003), (Chat.SUPERGROUP, -1003)]
     )
     def test_link_with_id(self, message, type_, id_):
         message.chat.username = None
@@ -792,7 +950,21 @@ class TestMessageWithoutRequest(TestMessageBase):
         # The leading - for group ids/ -100 for supergroup ids isn't supposed to be in the link
         assert message.link == f"https://t.me/c/{3}/{message.message_id}"
 
-    @pytest.mark.parametrize("id_, username", argvalues=[(None, "username"), (-3, None)])
+    def test_link_with_topics(self, message):
+        message.chat.username = None
+        message.chat.id = -1003
+        message.is_topic_message = True
+        message.message_thread_id = 123
+        assert message.link == f"https://t.me/c/3/{message.message_id}?thread=123"
+
+    def test_link_with_reply(self, message):
+        message.chat.username = None
+        message.chat.id = -1003
+        message.reply_to_message = Message(7, self.from_user, self.date, self.chat, text="Reply")
+        message.message_thread_id = 123
+        assert message.link == f"https://t.me/c/3/{message.message_id}?thread=123"
+
+    @pytest.mark.parametrize(("id_", "username"), argvalues=[(None, "username"), (-3, None)])
     def test_link_private_chats(self, message, id_, username):
         message.chat.type = Chat.PRIVATE
         message.chat.id = id_
@@ -901,7 +1073,7 @@ class TestMessageWithoutRequest(TestMessageBase):
             "[links](http://github.com/abc\\\\\\)def), "
             "[text\\-mention](tg://user?id=123456789) and ```\\`\\\\pre```\\. "
             r"http://google\.com and _bold *nested in ~strk\>trgh~ nested in* italic_\. "
-            "```python\nPython pre```\\. ||Spoiled||\\."
+            "```python\nPython pre```\\. ||Spoiled||\\. ![👍](tg://emoji?id=1)\\."
         )
 
         async def make_assertion(*_, **kwargs):
@@ -941,7 +1113,8 @@ class TestMessageWithoutRequest(TestMessageBase):
             r"<pre>`\pre</pre>. http://google.com "
             "and <i>bold <b>nested in <s>strk&gt;trgh</s> nested in</b> italic</i>. "
             '<pre><code class="python">Python pre</code></pre>. '
-            '<span class="tg-spoiler">Spoiled</span>.'
+            '<span class="tg-spoiler">Spoiled</span>. '
+            '<tg-emoji emoji-id="1">👍</tg-emoji>.'
         )
 
         async def make_assertion(*_, **kwargs):
@@ -1324,7 +1497,7 @@ class TestMessageWithoutRequest(TestMessageBase):
             quote=True,
         )
 
-    @pytest.mark.parametrize("disable_notification,protected", [(False, True), (True, False)])
+    @pytest.mark.parametrize(("disable_notification", "protected"), [(False, True), (True, False)])
     async def test_forward(self, monkeypatch, message, disable_notification, protected):
         async def make_assertion(*_, **kwargs):
             chat_id = kwargs["chat_id"] == 123456
@@ -1346,7 +1519,7 @@ class TestMessageWithoutRequest(TestMessageBase):
         )
         assert not await message.forward(635241)
 
-    @pytest.mark.parametrize("disable_notification,protected", [(True, False), (False, True)])
+    @pytest.mark.parametrize(("disable_notification", "protected"), [(True, False), (False, True)])
     async def test_copy(self, monkeypatch, message, disable_notification, protected):
         keyboard = [[1, 2]]
 
@@ -1387,7 +1560,7 @@ class TestMessageWithoutRequest(TestMessageBase):
         )
         assert not await message.copy(635241)
 
-    @pytest.mark.parametrize("disable_notification,protected", [(True, False), (False, True)])
+    @pytest.mark.parametrize(("disable_notification", "protected"), [(True, False), (False, True)])
     async def test_reply_copy(self, monkeypatch, message, disable_notification, protected):
         keyboard = [[1, 2]]
 
