@@ -154,6 +154,7 @@ BACKWARDS_COMPAT_KWARGS = {
         "thumb_url",
     },
     "InlineQueryResult(Game|Gif|Mpeg4Gif)": {"thumb_mime_type"},
+    "answer_inline_query": {"switch_pm_text", "switch_pm_parameter"},
 }
 
 
@@ -167,9 +168,10 @@ IGNORED_PARAM_REQUIREMENTS.update(BACKWARDS_COMPAT_KWARGS)
 def find_next_sibling_until(tag, name, until):
     for sibling in tag.next_siblings:
         if sibling is until:
-            return
+            return None
         if sibling.name == name:
             return sibling
+    return None
 
 
 def parse_table(h4) -> List[List[str]]:
@@ -280,10 +282,7 @@ def check_object(h4):
 def is_parameter_required_by_tg(field: str) -> bool:
     if field in {"Required", "Yes"}:
         return True
-    if field.split(".", 1)[0] == "Optional":  # splits the sentence and extracts first word
-        return False
-    else:
-        return True
+    return field.split(".", 1)[0] != "Optional"  # splits the sentence and extracts first word
 
 
 def check_required_param(
@@ -304,7 +303,7 @@ def check_required_param(
 
 
 def check_defaults_type(ptb_param: inspect.Parameter) -> bool:
-    return True if DefaultValue.get_value(ptb_param.default) is None else False
+    return DefaultValue.get_value(ptb_param.default) is None
 
 
 to_run = env_var_2_bool(os.getenv("TEST_OFFICIAL"))
